window.Gleam.initSearch([{"doc":"valid","title":"valid","content":"# Valid\n\n![CI](https://github.com/sporto/gleam-valid/workflows/test/badge.svg?branch=main)\n\nA validation library for [Gleam](https://gleam.run/).\n\nAPI Docs: &lt;https://hexdocs.pm/valid&gt;.\n\nThis library follows the principle [Parse don&#39;t validate](https://lexi-lambda.github.io/blog/2019/11/05/parse-don-t-validate/).\n\n```gleam\nfn user_validator(user: InputUser) -&gt; ValidatorResult(ValidUser, String) {\n  valid.build3(ValidUser)\n  |&gt; valid.check(user.name, valid.is_some(&quot;Please provide a name&quot;))\n  |&gt; valid.check(user.email, valid.is_some(&quot;Please provide an email&quot;))\n  |&gt; valid.check(user.age, valid.ok())\n}\n\ncase user_valid(input) {\n  Ok(valid_user) -&gt; ...\n  Error(errors) -&gt; ...\n}\n```\n\n## Install\n\n```\ngleam add valid\n```\n\n## Usage and Examples\n\n- For basic usage see &lt;test/basic_test.gleam&gt;\n\n### Validators\n\n- For string validators, see &lt;test/validator_string_test.gleam&gt;\n- For int validators, see &lt;test/validator_int_test.gleam&gt;\n- For list validators, see &lt;test/validator_list_test.gleam&gt;\n- For optional validators, see &lt;test/validator_option_test.gleam&gt;\n- For creating a custom validator, see &lt;test/validator_custom_test.gleam&gt;\n\n### Composition\n\n- For composing validators, see &lt;test/composition_test.gleam&gt;\n\n### Other\n\n- For validating a dictionary, see &lt;test/dictionary_test.gleam&gt;\n- For custom error types, see &lt;test/custom_error_test.gleam&gt;\n- For validating a whole structure, see &lt;test/whole_test.gleam&gt;\n","url":"index.html"},{"doc":"valid","title":"NonEmptyList","content":"pub type NonEmptyList(a) =\n  non_empty_list.NonEmptyList(a)\n A non empty list.\n Errors returned by a validator are returned in this format.\n\nSynonyms:\nvalid.NonEmptyList\nvalid NonEmptyList","url":"valid.html#NonEmptyList"},{"doc":"valid","title":"Validator","content":"pub type Validator(input, output, error) =\n  fn(input) -&gt; ValidatorResult(output, error)\n A Validator is a function that takes an input and\n returns a ValidatorResult\n\nSynonyms:\nvalid.Validator\nvalid Validator","url":"valid.html#Validator"},{"doc":"valid","title":"ValidatorResult","content":"pub type ValidatorResult(output, error) =\n  Result(output, NonEmptyList(error))\n\n\nSynonyms:\nvalid.ValidatorResult\nvalid ValidatorResult","url":"valid.html#ValidatorResult"},{"doc":"valid","title":"all","content":"pub fn all(\n  validators: List(fn(a) -&gt; Result(a, NonEmptyList(b))),\n) -&gt; fn(a) -&gt; Result(a, NonEmptyList(b))\n Validate a value using a list of validators.\n This runs all the validators in the list.\n\n The initial input is passed to all validators.\n All these validators must have the same input and output types.\n\n Returns Ok when all validators pass.\n Returns Error when any validator fails. The error will have all failures.\n\n ## Example\n\n See &lt;test/composition_test.gleam&gt;\n\nSynonyms:\nvalid.all\nvalid all","url":"valid.html#all"},{"doc":"valid","title":"build1","content":"pub fn build1(constructor: a) -&gt; Result(a, b)\n Build a validator for a type that has one attribute\n\n ## Example\n\n\ttype Person { Person(name: String) }\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build1(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t}\nSynonyms:\nvalid.build1\nvalid build1","url":"valid.html#build1"},{"doc":"valid","title":"build2","content":"pub fn build2(\n  constructor: fn(a, b) -&gt; c,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; c, d)\n Build a validator for a type that has two attributes\n\n ## Example\n\n\ttype Person { Person(name: String, age: Int) }\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build2(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t\t|&gt; valid.check(person.age, ...)\n\t}\nSynonyms:\nvalid.build2\nvalid build2","url":"valid.html#build2"},{"doc":"valid","title":"build3","content":"pub fn build3(\n  constructor: fn(a, b, c) -&gt; d,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; d, e)\n Build a validator for a type that has three attributes\n\n ## Example\n\n\ttype Person { Person(name: String, age: Int, email: String) }\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build3(person)\n\t\t|&gt; valid.check(person.name, name_validator)\n\t\t|&gt; valid.check(person.age, ...)\n\t\t|&gt; valid.check(person.email, ...)\n\t}\nSynonyms:\nvalid.build3\nvalid build3","url":"valid.html#build3"},{"doc":"valid","title":"build4","content":"pub fn build4(\n  constructor: fn(a, b, c, d) -&gt; e,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; e, f)\n Build a validator for a type that has four attributes\nSynonyms:\nvalid.build4\nvalid build4","url":"valid.html#build4"},{"doc":"valid","title":"build5","content":"pub fn build5(\n  constructor: fn(a, b, c, d, e) -&gt; f,\n) -&gt; Result(fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; f, g)\n Build a validator for a type that has five attributes\nSynonyms:\nvalid.build5\nvalid build5","url":"valid.html#build5"},{"doc":"valid","title":"build6","content":"pub fn build6(\n  constructor: fn(a, b, c, d, e, f) -&gt; g,\n) -&gt; Result(\n  fn(a) -&gt; fn(b) -&gt; fn(c) -&gt; fn(d) -&gt; fn(e) -&gt; fn(f) -&gt; g,\n  h,\n)\n Build a validator for a type that has six attributes\nSynonyms:\nvalid.build6\nvalid build6","url":"valid.html#build6"},{"doc":"valid","title":"check","content":"pub fn check(\n  accumulator: Result(fn(a) -&gt; b, NonEmptyList(c)),\n  input: d,\n  validator: fn(d) -&gt; Result(a, NonEmptyList(c)),\n) -&gt; Result(b, NonEmptyList(c))\n Validate an attribute.\n\n ## Example\n\n\tlet validator = fn(person: Person) {\n\t\tvalid.build1(Person)\n\t\t|&gt; valid.check(person.name, valid.string_is_not_empty(ErrorEmpty))\n\t}\n\nSynonyms:\nvalid.check\nvalid check","url":"valid.html#check"},{"doc":"valid","title":"check_only","content":"pub fn check_only(\n  accumulator: Result(fn(a) -&gt; b, NonEmptyList(c)),\n  input: d,\n  validator: fn(d) -&gt; Result(d, NonEmptyList(c)),\n) -&gt; Result(fn(a) -&gt; b, NonEmptyList(c))\n Performs a validation, but discards the resulting type.\n So the resulting type is not passed to the final constructor.\nSynonyms:\nvalid.check_only\nvalid check_only","url":"valid.html#check_only"},{"doc":"valid","title":"check_whole","content":"pub fn check_whole(\n  accumulator: Result(a, NonEmptyList(b)),\n  validator: fn(a) -&gt; Result(c, NonEmptyList(b)),\n) -&gt; Result(c, NonEmptyList(b))\n Validate the resulting type as a whole.\n\n Sometimes we need to validate a property in relation to another.\n This validator must be at the end of the pipeline, so it receives the final type.\n\n For example see &lt;test/check_whole_test.gleam&gt;\n\nSynonyms:\nvalid.check_whole\nvalid check_whole","url":"valid.html#check_whole"},{"doc":"valid","title":"if_some","content":"pub fn if_some(\n  validator: fn(a) -&gt; Result(b, NonEmptyList(c)),\n) -&gt; fn(Option(a)) -&gt; Result(Option(b), NonEmptyList(c))\n Validate an optional value.\n\n Run the validator only if the value is Some.\n If the value is None then return None back.\n\n ## Example\n\n See &lt;test/validator_optional_test.gleam&gt;\n\nSynonyms:\nvalid.if_some\nvalid if_some","url":"valid.html#if_some"},{"doc":"valid","title":"int_max","content":"pub fn int_max(\n  max: Int,\n  error: a,\n) -&gt; fn(Int) -&gt; Result(Int, NonEmptyList(a))\n\nSynonyms:\nvalid.int_max\nvalid int_max","url":"valid.html#int_max"},{"doc":"valid","title":"int_min","content":"pub fn int_min(\n  min: Int,\n  error: a,\n) -&gt; fn(Int) -&gt; Result(Int, NonEmptyList(a))\n Integer checks\nSynonyms:\nvalid.int_min\nvalid int_min","url":"valid.html#int_min"},{"doc":"valid","title":"is_some","content":"pub fn is_some(\n  error: a,\n) -&gt; fn(Option(b)) -&gt; Result(b, NonEmptyList(a))\n Option checks\n\n Validate that a value is not None.\n Returns the value if Some.\n\n ## Example\n\n See &lt;test/validator_option_test.gleam\n\nSynonyms:\nvalid.is_some\nvalid is_some","url":"valid.html#is_some"},{"doc":"valid","title":"keep","content":"pub fn keep(\n  accumulator: Result(fn(a) -&gt; b, NonEmptyList(c)),\n  value: a,\n) -&gt; Result(b, NonEmptyList(c))\n Keep a value as is\n\n ## Example\n\n See &lt;test/check_other_test.gleam\n\nSynonyms:\nvalid.keep\nvalid keep","url":"valid.html#keep"},{"doc":"valid","title":"list_every","content":"pub fn list_every(\n  validator: fn(a) -&gt; Result(b, NonEmptyList(c)),\n) -&gt; fn(List(a)) -&gt; Result(List(b), NonEmptyList(c))\n Validate a list of items.\n\n Run the given validator for each item returning all the errors.\n\n ## Example\n\n See &lt;test/composition_test.gleam\n\nSynonyms:\nvalid.list_every\nvalid list_every","url":"valid.html#list_every"},{"doc":"valid","title":"list_is_not_empty","content":"pub fn list_is_not_empty(\n  error: a,\n) -&gt; fn(List(b)) -&gt; Result(List(b), NonEmptyList(a))\n List checks\n\n Validate that a list is not empty\nSynonyms:\nvalid.list_is_not_empty\nvalid list_is_not_empty","url":"valid.html#list_is_not_empty"},{"doc":"valid","title":"list_max_length","content":"pub fn list_max_length(\n  max: Int,\n  error: a,\n) -&gt; fn(List(b)) -&gt; Result(List(b), NonEmptyList(a))\n Validate the max number of items in a list\nSynonyms:\nvalid.list_max_length\nvalid list_max_length","url":"valid.html#list_max_length"},{"doc":"valid","title":"list_min_length","content":"pub fn list_min_length(\n  min: Int,\n  error: a,\n) -&gt; fn(List(b)) -&gt; Result(List(b), NonEmptyList(a))\n Validate the min number of items in a list\nSynonyms:\nvalid.list_min_length\nvalid list_min_length","url":"valid.html#list_min_length"},{"doc":"valid","title":"ok","content":"pub fn ok() -&gt; fn(a) -&gt; Result(a, NonEmptyList(b))\n A validator that always succeeds\nSynonyms:\nvalid.ok\nvalid ok","url":"valid.html#ok"},{"doc":"valid","title":"optional_in","content":"pub fn optional_in(\n  get: fn(a) -&gt; Result(b, c),\n) -&gt; fn(a) -&gt; Result(Option(b), NonEmptyList(d))\n\nSynonyms:\nvalid.optional_in\nvalid optional_in","url":"valid.html#optional_in"},{"doc":"valid","title":"optional_in_dict","content":"pub fn optional_in_dict(\n  key: a,\n) -&gt; fn(Dict(a, b)) -&gt; Result(Option(b), NonEmptyList(c))\n Validate an optional attribute in a dict\n\n ## Example\n\n See &lt;test/validator_optional_test.gleam&gt;\n\nSynonyms:\nvalid.optional_in_dict\nvalid optional_in_dict","url":"valid.html#optional_in_dict"},{"doc":"valid","title":"required_in","content":"pub fn required_in(\n  get: fn(a) -&gt; Result(b, c),\n  error: d,\n) -&gt; fn(a) -&gt; Result(b, NonEmptyList(d))\n Validate an attribute required in a data type\n Here you provide your own accessor\n The accessor should return a `Result`\n\n ## Example\n\n See &lt;test/validator_required_test.gleam&gt;\n\nSynonyms:\nvalid.required_in\nvalid required_in","url":"valid.html#required_in"},{"doc":"valid","title":"required_in_dict","content":"pub fn required_in_dict(\n  key: String,\n  error: a,\n) -&gt; fn(Dict(String, b)) -&gt; Result(b, NonEmptyList(a))\n Validate an attribute required in a dictionary\n If you have a dictionary instead of a custom type, use this.\n\n ## Example\n\n See &lt;test/dictionary_test.gleam&gt;\n\nSynonyms:\nvalid.required_in_dict\nvalid required_in_dict","url":"valid.html#required_in_dict"},{"doc":"valid","title":"string_is_email","content":"pub fn string_is_email(\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, NonEmptyList(a))\n Validate if a string is an email.\n\n This checks if a string follows a simple pattern `_@_`.\nSynonyms:\nvalid.string_is_email\nvalid string_is_email","url":"valid.html#string_is_email"},{"doc":"valid","title":"string_is_float","content":"pub fn string_is_float(\n  error: a,\n) -&gt; fn(String) -&gt; Result(Float, NonEmptyList(a))\n Validate if a string parses to an Float. Returns the Float if so.\nSynonyms:\nvalid.string_is_float\nvalid string_is_float","url":"valid.html#string_is_float"},{"doc":"valid","title":"string_is_int","content":"pub fn string_is_int(\n  error: a,\n) -&gt; fn(String) -&gt; Result(Int, NonEmptyList(a))\n Validate if a string parses to an Int. Returns the Int if so.\nSynonyms:\nvalid.string_is_int\nvalid string_is_int","url":"valid.html#string_is_int"},{"doc":"valid","title":"string_is_not_empty","content":"pub fn string_is_not_empty(\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, NonEmptyList(a))\n String checks\n\n Validate that a string is not empty\nSynonyms:\nvalid.string_is_not_empty\nvalid string_is_not_empty","url":"valid.html#string_is_not_empty"},{"doc":"valid","title":"string_max_length","content":"pub fn string_max_length(\n  max: Int,\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, NonEmptyList(a))\n Validate the max length of a string\nSynonyms:\nvalid.string_max_length\nvalid string_max_length","url":"valid.html#string_max_length"},{"doc":"valid","title":"string_min_length","content":"pub fn string_min_length(\n  min: Int,\n  error: a,\n) -&gt; fn(String) -&gt; Result(String, NonEmptyList(a))\n Validate the min length of a string\nSynonyms:\nvalid.string_min_length\nvalid string_min_length","url":"valid.html#string_min_length"},{"doc":"valid","title":"then","content":"pub fn then(\n  validator1: fn(a) -&gt; Result(b, NonEmptyList(c)),\n  validator2: fn(b) -&gt; Result(d, NonEmptyList(c)),\n) -&gt; fn(a) -&gt; Result(d, NonEmptyList(c))\n Compose validators\n\n Run the first validator and if successful then the second.\n This short circuits, so only returns the first error.\n\n ## Example\n\n See &lt;test/composition_test.gleam&gt;\n\nSynonyms:\nvalid.then\nvalid then","url":"valid.html#then"},{"doc":"valid","title":"valid","content":"","url":"valid.html"}]);